@@ -84,6 +84,7 @@ void PlasmoidProtocol::init()
     }
 
     m_containment = new Plasma::Containment(m_systrayApplet, "null", containmentId);
+    m_containment->setImmutability(Plasma::Types::Mutable);
     m_containment->setFormFactor(Plasma::Types::Horizontal);
     m_containment->setLocation(m_systrayApplet->location());
     m_containment->init();
@@ -238,15 +239,19 @@ void PlasmoidProtocol::initDBusActivatables()
     QDBusPendingCall async = QDBusConnection::sessionBus().interface()->asyncCall("ListNames");
     QDBusPendingCallWatcher *callWatcher = new QDBusPendingCallWatcher(async, this);
     connect(callWatcher, &QDBusPendingCallWatcher::finished,
-            this,        &PlasmoidProtocol::serviceNameFetchFinished);
+            [=](QDBusPendingCallWatcher *callWatcher){
+                PlasmoidProtocol::serviceNameFetchFinished(callWatcher, QDBusConnection::sessionBus());
+            });
 
     QDBusPendingCall systemAsync = QDBusConnection::systemBus().interface()->asyncCall("ListNames");
     QDBusPendingCallWatcher *systemCallWatcher = new QDBusPendingCallWatcher(systemAsync, this);
     connect(systemCallWatcher, &QDBusPendingCallWatcher::finished,
-            this,        &PlasmoidProtocol::serviceNameFetchFinished);
+            [=](QDBusPendingCallWatcher *callWatcher){
+                PlasmoidProtocol::serviceNameFetchFinished(callWatcher, QDBusConnection::systemBus());
+            });
 }
 
-void PlasmoidProtocol::serviceNameFetchFinished(QDBusPendingCallWatcher* watcher)
+void PlasmoidProtocol::serviceNameFetchFinished(QDBusPendingCallWatcher* watcher, const QDBusConnection &connection)
 {
     QDBusPendingReply<QStringList> propsReply = *watcher;
     watcher->deleteLater();
@@ -265,13 +270,15 @@ void PlasmoidProtocol::serviceNameFetchFinished(QDBusPendingCallWatcher* watcher
     // This makes mpris work, since it wants to match org.mpris.MediaPlayer2.dragonplayer
     // against org.mpris.MediaPlayer2
     QDBusServiceWatcher *serviceWatcher = new QDBusServiceWatcher(QString(),
-                                                QDBusConnection::sessionBus(),
+                                                connection,
                                                 QDBusServiceWatcher::WatchForOwnerChange,
                                                 this);
     connect(serviceWatcher, &QDBusServiceWatcher::serviceRegistered, this, &PlasmoidProtocol::serviceRegistered);
     connect(serviceWatcher, &QDBusServiceWatcher::serviceUnregistered, this, &PlasmoidProtocol::serviceUnregistered);
 }
 
+
+
 void PlasmoidProtocol::serviceRegistered(const QString &service)
 {
     foreach (const QString &plugin, m_dbusActivatableTasks.keys()) {
