@@ -26,6 +26,7 @@
 #include <zlib.h>
 
 #include <QDebug>
+#include <QDir>
 #include <QDialog>
 #include <QMenu>
 #include <QDBusConnection>
@@ -51,11 +52,10 @@
 #include <prison/QRCodeBarcode>
 #endif
 
+#include <config-X11.h>
 #if HAVE_X11
 #include <QX11Info>
-#include <X11/Xlib.h>
-#include <X11/Xatom.h>
-#include <fixx11h.h>
+#include <xcb/xcb.h>
 #endif
 
 //#define NOISY_KLIPPER
@@ -400,6 +400,15 @@ void Klipper::saveHistory(bool empty) {
     // don't use "appdata", klipper is also a kicker applet
     QString history_file_name(QStandardPaths::locate(QStandardPaths::GenericDataLocation,
                                                      QStringLiteral("klipper/history2.lst")));
+    if ( history_file_name.isNull() || history_file_name.isEmpty() ) {
+        // try creating the file
+        QDir dir(QStandardPaths::writableLocation(QStandardPaths::GenericDataLocation));
+        if (!dir.mkpath(QStringLiteral("klipper"))) {
+            qWarning() << failed_save_warning ;
+            return;
+        }
+        history_file_name = dir.absoluteFilePath(QStringLiteral("klipper/history2.lst"));
+    }
     if ( history_file_name.isNull() || history_file_name.isEmpty() ) {
         qWarning() << failed_save_warning ;
         return;
@@ -586,13 +595,17 @@ bool Klipper::blockFetchingNewData()
 //   selected when Klipper asked first.
 // Use XQueryPointer rather than QApplication::mouseButtons()/keyboardModifiers(), because
 //   Klipper needs the very current state.
-    Window root, child;
-    int root_x, root_y, win_x, win_y;
-    uint state;
-    XQueryPointer( QX11Info::display(), QX11Info::appRootWindow(), &root, &child,
-                   &root_x, &root_y, &win_x, &win_y, &state );
-    if( ( state & ( ShiftMask | Button1Mask )) == ShiftMask // #85198
-        || ( state & Button1Mask ) == Button1Mask ) { // #80302
+    if (!QX11Info::isPlatformX11()) {
+        return false;
+    }
+    xcb_connection_t *c = QX11Info::connection();
+    const xcb_query_pointer_cookie_t cookie = xcb_query_pointer_unchecked(c, QX11Info::appRootWindow());
+    QScopedPointer<xcb_query_pointer_reply_t, QScopedPointerPodDeleter> queryPointer(xcb_query_pointer_reply(c, cookie, nullptr));
+    if (queryPointer.isNull()) {
+        return false;
+    }
+    if (((queryPointer->mask & (XCB_KEY_BUT_MASK_SHIFT | XCB_KEY_BUT_MASK_BUTTON_1)) == XCB_KEY_BUT_MASK_SHIFT) // BUG: 85198
+            || ((queryPointer->mask & XCB_KEY_BUT_MASK_BUTTON_1) == XCB_KEY_BUT_MASK_BUTTON_1)) { // BUG: 80302
         m_pendingContentsCheck = true;
         m_pendingCheckTimer.start( 100 );
         return true;
@@ -814,7 +827,11 @@ bool Klipper::ignoreClipboardChanges() const
 
 void Klipper::updateTimestamp()
 {
-    QX11Info::setAppTime(QX11Info::getTimestamp());
+#if HAVE_X11
+    if (QX11Info::isPlatformX11()) {
+        QX11Info::setAppTime(QX11Info::getTimestamp());
+    }
+#endif
 }
 
 void Klipper::slotEditData()
