@@ -33,6 +33,8 @@
 #include <KGlobalAccel>
 #include <QAction>
 #include <KLocalizedString>
+#include <KDirWatch>
+#include <KCrash/KCrash>
 
 #include <kdeclarative/qmlobject.h>
 
@@ -51,6 +53,12 @@ View::View(QWindow *parent)
     setColor(QColor(Qt::transparent));
     setFlags(Qt::FramelessWindowHint);
 
+    KCrash::setFlags(KCrash::AutoRestart);
+
+    m_config = KConfigGroup(KSharedConfig::openConfig("krunnerrc"), "General");
+
+    setFreeFloating(m_config.readEntry("FreeFloating", false));
+
     new AppAdaptor(this);
     QDBusConnection::sessionBus().registerObject(QLatin1String("/App"), this);
 
@@ -60,7 +68,7 @@ View::View(QWindow *parent)
         a->setText(i18n("Run Command"));
         a->setObjectName("run command");
         KGlobalAccel::self()->setDefaultShortcut(a, QList<QKeySequence>() << QKeySequence(Qt::ALT + Qt::Key_Space), KGlobalAccel::NoAutoloading);
-        KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << QKeySequence(Qt::ALT + Qt::Key_Space) << QKeySequence(Qt::ALT + Qt::Key_F2));
+        KGlobalAccel::self()->setShortcut(a, QList<QKeySequence>() << QKeySequence(Qt::ALT + Qt::Key_Space) << QKeySequence(Qt::ALT + Qt::Key_F2) << Qt::Key_Search);
 
         a = new QAction(0);
         QObject::connect(a, SIGNAL(triggered(bool)), SLOT(displayWithClipboardContents()));
@@ -94,6 +102,12 @@ View::View(QWindow *parent)
 
     connect(this, SIGNAL(visibleChanged(bool)), this, SLOT(resetScreenPos()));
 
+    KDirWatch::self()->addFile(m_config.name());
+
+    // Catch both, direct changes to the config file ...
+    connect(KDirWatch::self(), &KDirWatch::dirty, this, &View::reloadConfig);
+    connect(KDirWatch::self(), &KDirWatch::created, this, &View::reloadConfig);
+
     if (m_qmlObj->rootObject()) {
         connect(m_qmlObj->rootObject(), SIGNAL(widthChanged()), this, SLOT(resetScreenPos()));
     }
@@ -109,6 +123,33 @@ View::~View()
 {
 }
 
+bool View::freeFloating() const
+{
+    return m_floating;
+}
+
+void View::setFreeFloating(bool floating)
+{
+    if (m_floating == floating) {
+        return;
+    }
+
+    m_floating = floating;
+    if (m_floating) {
+        setLocation(Plasma::Types::Floating);
+    } else {
+        setLocation(Plasma::Types::TopEdge);
+    }
+
+    positionOnScreen();
+}
+
+void View::reloadConfig()
+{
+    m_config.config()->reparseConfiguration();
+    setFreeFloating(m_config.readEntry("FreeFloating", false));
+}
+
 bool View::event(QEvent *event)
 {
     // QXcbWindow overwrites the state in its show event. There are plans
@@ -188,7 +229,6 @@ void View::positionOnScreen()
         KWindowEffects::slideWindow(winId(), KWindowEffects::NoEdge, 0);
     } else {
         KWindowSystem::setOnAllDesktops(winId(), true);
-        KWindowSystem::setType(winId(), NET::Dock);
         KWindowEffects::slideWindow(winId(), KWindowEffects::TopEdge, 0);
     }
 
@@ -246,11 +286,4 @@ void View::switchUser()
     m_qmlObj->rootObject()->setProperty("query", "SESSIONS");
 }
 
-void View::initializeStartupNotification()
-{
-    //vHanda will pay me another dinner if he does not implement this one
-    return;
-}
-
-
 #include "moc_view.cpp"
