@@ -40,6 +40,9 @@ public:
 
     void positionOnScreen();
 
+    bool freeFloating() const;
+    void setFreeFloating(bool floating);
+
 protected:
     virtual bool event(QEvent* event);
     virtual void showEvent(QShowEvent *event);
@@ -51,16 +54,17 @@ public Q_SLOTS:
     void query(const QString &term);
     void querySingleRunner(const QString &runnerName, const QString &term);
     void switchUser();
-    void initializeStartupNotification();
 
 protected Q_SLOTS:
     void screenGeometryChanged();
     void resetScreenPos();
     void displayOrHide();
+    void reloadConfig();
 
 private:
     QPoint m_customPos;
     KDeclarative::QmlObject *m_qmlObj;
+    KConfigGroup m_config;
     qreal m_offset;
     bool m_floating : 1;
 };
