@@ -20,6 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 #include <QDateTime>
 #include <QCommandLineParser>
+#include <QSessionManager>
 
 #include <iostream>
 
@@ -50,11 +51,20 @@ static void signalHandler(int signum)
 
 int main(int argc, char* argv[])
 {
+    KLocalizedString::setApplicationDomain("kscreenlocker_greet");
+
     ScreenLocker::UnlockApp app(argc, argv);
     QCoreApplication::setApplicationName(QStringLiteral("kscreenlocker_greet"));
     QCoreApplication::setApplicationVersion(QStringLiteral("0.1"));
     QCoreApplication::setOrganizationDomain(QStringLiteral("kde.org"));
 
+    // disable session management for the greeter
+    auto disableSessionManagement = [](QSessionManager &sm) {
+        sm.setRestartHint(QSessionManager::RestartNever);
+    };
+    QObject::connect(&app, &QGuiApplication::commitDataRequest, disableSessionManagement);
+    QObject::connect(&app, &QGuiApplication::saveStateRequest, disableSessionManagement);
+
     QCommandLineParser parser;
     parser.setApplicationDescription(i18n("Greeter for the KDE Plasma Workspaces Screen locker"));
     parser.addHelpOption();
@@ -69,7 +79,7 @@ int main(int argc, char* argv[])
                                        QStringLiteral("milliseconds"),
                                        QStringLiteral("0"));
     QCommandLineOption nolockOption(QStringLiteral("nolock"),
-                                              i18n("Don't show any lock user interface."));
+                                    i18n("Don't show any lock user interface."));
 
     parser.addOption(testingOption);
     parser.addOption(immediateLockOption);
