@@ -29,11 +29,11 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
 ******************************************************************/
 
-#include <kglobalsettings.h>
 #include <QDir>
 #include <krun.h>
 #include <config-workspace.h>
 #include <config-unix.h> // HAVE_LIMITS_H
+#include <config-ksmserver.h>
 
 #include <pwd.h>
 #include <sys/types.h>
@@ -57,17 +57,13 @@ CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include <limits.h>
 #endif
 
-#include <QPushButton>
 #include <QTimer>
 #include <QtDBus/QtDBus>
 
-#include <klocale.h>
-#include <kglobal.h>
 #include <kconfig.h>
-#include <kapplication.h>
-#include <ktemporaryfile.h>
 #include <knotification.h>
 #include <kconfiggroup.h>
+#include <KSharedConfig>
 #include <kprocess.h>
 
 #include "global.h"
@@ -200,11 +196,11 @@ void KSMServer::wmProcessChange()
     if( wmProcess->state() == QProcess::NotRunning )
     { // wm failed to launch for some reason, go with kwin instead
         kWarning( 1218 ) << "Window manager" << wm << "failed to launch";
-        if( wm == QStringLiteral( "kwin" ) )
+        if( wm == QStringLiteral( KWIN_BIN ) )
             return; // uhoh, kwin itself failed
         kDebug( 1218 ) << "Launching KWin";
-        wm = QStringLiteral( "kwin" );
-        wmCommands = ( QStringList() << QStringLiteral( "kwin" ) );
+        wm = QStringLiteral( KWIN_BIN );
+        wmCommands = ( QStringList() << QStringLiteral( KWIN_BIN ) );
         // launch it
         launchWM( QList< QStringList >() << wmCommands );
         return;
@@ -427,7 +423,7 @@ void KSMServer::runUserAutostart()
     // now let's execute all the stuff in the autostart folder.
     // the stuff will actually be really executed when the event loop is
     // entered, since KRun internally uses a QTimer
-    QDir dir(QStandardPaths::locate( QStandardPaths::GenericConfigLocation, QStringLiteral("autostart"), QStandardPaths::LocateDirectory));
+    QDir dir(QStandardPaths::writableLocation(QStandardPaths::GenericConfigLocation) + QDir::separator() + QStringLiteral("autostart"));
     if (dir.exists()) {
         const QStringList entries = dir.entryList( QDir::Files );
         foreach (const QString& file, entries) {
