@@ -67,10 +67,11 @@ public:
           abstractSortingStrategy(0),
           groupIsFullLimit(0),
           showOnlyCurrentDesktop(false),
-          showOnlyCurrentActivity(false),
+          showOnlyCurrentActivity(true),
           showOnlyCurrentScreen(false),
           showOnlyMinimized(false),
           onlyGroupWhenFull(false),
+          cachedOnlyGroupWhenFull(false),
           changingGroupingStrategy(false),
           readingLauncherConfig(false),
           separateLaunchers(true),
@@ -131,6 +132,7 @@ public:
     bool showOnlyCurrentScreen : 1;
     bool showOnlyMinimized : 1;
     bool onlyGroupWhenFull : 1;
+    bool cachedOnlyGroupWhenFull : 1;
     bool changingGroupingStrategy : 1;
     bool readingLauncherConfig : 1;
     bool separateLaunchers : 1;
@@ -170,8 +172,14 @@ GroupManager::GroupManager(QObject *parent)
 GroupManager::~GroupManager()
 {
     TaskManager::self()->setTrackGeometry(false, d->configToken);
+
     delete d->abstractSortingStrategy;
-    delete d->abstractGroupingStrategy;
+
+    if (d->abstractGroupingStrategy) {
+        d->abstractGroupingStrategy->setDestroyGroupsOnDestruction(false);
+        delete d->abstractGroupingStrategy;
+    }
+
     delete d;
 }
 
@@ -1070,7 +1078,7 @@ int GroupManager::fullLimit() const
 void GroupManager::setFullLimit(int limit)
 {
     //kDebug() << limit;
-    if (!d->groupIsFullLimit != limit) {
+    if (d->groupIsFullLimit != limit) {
         d->groupIsFullLimit = limit;
 
         if (d->onlyGroupWhenFull) {
@@ -1239,11 +1247,12 @@ AbstractGroupingStrategy* GroupManager::taskGrouper() const
 
 void GroupManager::setGroupingStrategy(TaskGroupingStrategy strategy)
 {
-    if (d->changingGroupingStrategy || d->groupingStrategy == strategy) {
+    if (d->changingGroupingStrategy || (d->cachedOnlyGroupWhenFull == d->onlyGroupWhenFull && d->groupingStrategy == strategy)) {
         return;
     }
 
     d->changingGroupingStrategy = true;
+    d->cachedOnlyGroupWhenFull = d->onlyGroupWhenFull;
 
     //kDebug() << strategy << kBacktrace();
     if (d->onlyGroupWhenFull) {
@@ -1253,7 +1262,7 @@ void GroupManager::setGroupingStrategy(TaskGroupingStrategy strategy)
 
     if (d->abstractGroupingStrategy) {
         disconnect(d->abstractGroupingStrategy, 0, this, 0);
-        d->abstractGroupingStrategy->destroy();
+        delete d->abstractGroupingStrategy;
         d->abstractGroupingStrategy = 0;
     }
 
