@@ -20,10 +20,12 @@
 #include "containmentconfigview.h"
 #include "shellcorona.h"
 #include "shellmanager.h"
+#include "krunner_interface.h"
 
 #include <QQmlEngine>
 #include <QQmlContext>
 #include <QScreen>
+#include <qopenglshaderprogram.h>
 
 #include <kwindowsystem.h>
 #include <klocalizedstring.h>
@@ -32,6 +34,7 @@
 
 DesktopView::DesktopView(ShellCorona *corona, QScreen *screen)
     : PlasmaQuick::View(corona, 0),
+      m_corona(corona),
       m_stayBehind(false),
       m_fillScreen(false),
       m_dashboardShown(false)
@@ -47,6 +50,16 @@ DesktopView::DesktopView(ShellCorona *corona, QScreen *screen)
 
     QObject::connect(corona, &Plasma::Corona::packageChanged,
                      this, &DesktopView::coronaPackageChanged);
+
+    connect(this, &DesktopView::sceneGraphInitialized, this,
+        [this, corona]() {
+            // check whether the GL Context supports OpenGL
+            // Note: hasOpenGLShaderPrograms is broken, see QTBUG--39730
+            if (!QOpenGLShaderProgram::hasOpenGLShaderPrograms(openglContext())) {
+                qWarning() << "GLSL not available, Plasma won't be functional";
+                QMetaObject::invokeMethod(corona, "showOpenGLNotCompatibleWarning", Qt::QueuedConnection);
+            }
+        }, Qt::DirectConnection);
 }
 
 DesktopView::~DesktopView()
@@ -86,12 +99,6 @@ void DesktopView::setFillScreen(bool fillScreen)
     emit fillScreenChanged();
 }
 
-void DesktopView::moveEvent(QMoveEvent* ev)
-{
-    adaptToScreen();
-    QWindow::moveEvent(ev);
-}
-
 void DesktopView::adaptToScreen()
 {
     //This happens sometimes, when shutting down the process
@@ -102,11 +109,16 @@ void DesktopView::adaptToScreen()
         setGeometry(screen()->geometry());
         setMinimumSize(screen()->geometry().size());
         setMaximumSize(screen()->geometry().size());
+
+        disconnect(m_oldScreen.data(), &QScreen::geometryChanged, this, static_cast<void (QWindow::*)(const QRect&)>(&QWindow::setGeometry));
         connect(screen(), &QScreen::geometryChanged, this, static_cast<void (QWindow::*)(const QRect&)>(&QWindow::setGeometry), Qt::UniqueConnection);
+
     } else {
         disconnect(screen(), &QScreen::geometryChanged, this, static_cast<void (QWindow::*)(const QRect&)>(&QWindow::setGeometry));
     }
-//     qDebug() << "adapted" << geometry() << (containment () ? containment()->wallpaper() : "xx");
+
+    m_oldScreen = screen();
+     qDebug() << "adapted" << geometry() << (containment () ? containment()->wallpaper() : "xx");
 }
 
 void DesktopView::ensureStayBehind()
@@ -135,10 +147,6 @@ void DesktopView::setDashboardShown(bool shown)
         KWindowSystem::raiseWindow(winId());
         KWindowSystem::forceActiveWindow(winId());
 
-        QObject *wpGraphicObject = containment()->property("wallpaperGraphicsObject").value<QObject *>();
-        if (wpGraphicObject) {
-            wpGraphicObject->setProperty("opacity", 0.3);
-        }
     } else {
         if (m_stayBehind) {
             KWindowSystem::setType(winId(), NET::Desktop);
@@ -147,13 +155,10 @@ void DesktopView::setDashboardShown(bool shown)
         lower();
         KWindowSystem::lowerWindow(winId());
 
-        QObject *wpGraphicObject = containment()->property("wallpaperGraphicsObject").value<QObject *>();
-        if (wpGraphicObject) {
-            wpGraphicObject->setProperty("opacity", 1);
-        }
     }
 
     m_dashboardShown = shown;
+    emit dashboardShownChanged();
 }
 
 bool DesktopView::event(QEvent *e)
@@ -177,6 +182,22 @@ bool DesktopView::event(QEvent *e)
     return PlasmaQuick::View::event(e);
 }
 
+void DesktopView::keyPressEvent(QKeyEvent *e)
+{
+    // When a key is pressed on desktop when nothing else is active forward the key to krunner
+    if (activeFocusItem() == contentItem()) {
+        const QString text = e->text().trimmed();
+        if (!text.isEmpty() && text[0].isPrint()) {
+            const QString interface("org.kde.krunner");
+            org::kde::krunner::App krunner(interface, "/App", QDBusConnection::sessionBus());
+            krunner.query(text);
+            e->accept();
+        }
+    }
+
+    QQuickView::keyPressEvent(e);
+}
+
 bool DesktopView::isDashboardShown() const
 {
     return m_dashboardShown;
@@ -211,4 +232,14 @@ void DesktopView::coronaPackageChanged(const Plasma::Package &package)
     setSource(QUrl::fromLocalFile(package.filePath("views", "Desktop.qml")));
 }
 
+void DesktopView::screenDestroyed(QObject* screen)
+{
+//     NOTE: this is overriding the screen destroyed slot, we need to do this because
+//     otherwise Qt goes mental and starts moving our panels. See:
+//     https://codereview.qt-project.org/#/c/88351/
+    if (screen == this->screen()) {
+        m_corona->removeScreen(this);
+    }
+}
+
 #include "moc_desktopview.cpp"
