@@ -31,6 +31,8 @@ class DesktopView : public PlasmaQuick::View
     Q_OBJECT
     Q_PROPERTY(bool stayBehind READ stayBehind WRITE setStayBehind NOTIFY stayBehindChanged)
     Q_PROPERTY(bool fillScreen READ fillScreen WRITE setFillScreen NOTIFY fillScreenChanged)
+    //the qml part doesn't need to be able to write it, hide for now
+    Q_PROPERTY(bool dashboardShown READ isDashboardShown NOTIFY dashboardShownChanged)
 
 public:
     explicit DesktopView(ShellCorona *corona, QScreen *screen);
@@ -49,23 +51,27 @@ public:
 
 protected:
     bool event(QEvent *e);
+    void keyPressEvent(QKeyEvent *e);
 
 protected Q_SLOTS:
     /**
      * It will be called when the configuration is requested
      */
     virtual void showConfigurationInterface(Plasma::Applet *applet);
+    void screenDestroyed(QObject* screen);
 
 Q_SIGNALS:
     void stayBehindChanged();
     void fillScreenChanged();
+    void dashboardShownChanged();
 
 private:
     void coronaPackageChanged(const Plasma::Package &package);
     void ensureStayBehind();
-    virtual void moveEvent(QMoveEvent* ev);
 
     QPointer<PlasmaQuick::ConfigView> m_configView;
+    QPointer<QScreen> m_oldScreen;
+    ShellCorona *m_corona;
     bool m_stayBehind : 1;
     bool m_fillScreen : 1;
     bool m_dashboardShown : 1;
