@@ -55,7 +55,8 @@ PanelView::PanelView(ShellCorona *corona, QWindow *parent)
        m_thickness(30),
        m_alignment(Qt::AlignLeft),
        m_corona(corona),
-       m_visibilityMode(NormalPanel)
+       m_visibilityMode(NormalPanel),
+       m_background(0)
 {
     setResizeMode(QQuickView::SizeRootObjectToView);
     setClearBeforeRendering(true);
@@ -78,18 +79,6 @@ PanelView::PanelView(ShellCorona *corona, QWindow *parent)
     connect(&m_unhideTimer, &QTimer::timeout,
             this, &PanelView::restoreAutoHide);
 
-    //Screen management
-    //When the screen is set, the screen is recreated internally, so we need to
-    //set anything that depends on the winId()
-    connect(this, &QWindow::screenChanged, this, [=]() {
-            if (!screen())
-                return;
-            KWindowSystem::setOnAllDesktops(winId(), true);
-            KWindowSystem::setType(winId(), NET::Dock);
-            setVisibilityMode(m_visibilityMode);
-            showTemporarily();
-        }
-    );
     connect(this, &QWindow::screenChanged,
             this, &PanelView::screenChangedProxy);
     //cannot use the new syntax as start() is overloaded
@@ -129,17 +118,6 @@ PanelView::PanelView(ShellCorona *corona, QWindow *parent)
 PanelView::~PanelView()
 {
     if (containment()) {
-        config().writeEntry("offset", m_offset);
-        config().writeEntry("max", m_maxLength);
-        config().writeEntry("min", m_minLength);
-        if (formFactor() == Plasma::Types::Vertical) {
-            config().writeEntry("length", size().height());
-            config().writeEntry("thickness", size().width());
-        } else {
-            config().writeEntry("length", size().width());
-            config().writeEntry("thickness", size().height());
-        }
-        config().writeEntry("alignment", (int)m_alignment);
         m_corona->requestApplicationConfigSync();
     }
     PanelShadows::self()->removeWindow(this);
@@ -353,8 +331,6 @@ void PanelView::setVisibilityMode(PanelView::VisibilityMode mode)
     } else {
         KWindowSystem::clearState(winId(), NET::KeepBelow);
     }
-    //life is vastly simpler if we ensure we're visible now
-    show();
 
     disconnect(containment(), &Plasma::Applet::activated, this, &PanelView::showTemporarily);
     if (!(mode == NormalPanel || mode == WindowsGoBelow || mode == AutoHide)) {
@@ -381,95 +357,116 @@ void PanelView::positionPanel()
         return;
     }
 
-    QScreen *s = screen();
-    QPoint position;
     containment()->reactToScreenChange();
+    KWindowEffects::SlideFromLocation slideLocation = KWindowEffects::NoEdge;
 
     switch (containment()->location()) {
     case Plasma::Types::TopEdge:
         containment()->setFormFactor(Plasma::Types::Horizontal);
-
-        switch (m_alignment) {
-        case Qt::AlignCenter:
-            position = QPoint(QPoint(s->geometry().center().x(), s->geometry().top()) + QPoint(m_offset - size().width()/2, m_distance));
-            break;
-        case Qt::AlignRight:
-            position = QPoint(s->geometry().topRight() - QPoint(m_offset + size().width(), m_distance));
-            break;
-        case Qt::AlignLeft:
-        default:
-            position = QPoint(s->geometry().topLeft() + QPoint(m_offset, m_distance));
-        }
+        slideLocation = KWindowEffects::TopEdge;
         break;
 
     case Plasma::Types::LeftEdge:
         containment()->setFormFactor(Plasma::Types::Vertical);
-
-        switch (m_alignment) {
-        case Qt::AlignCenter:
-            position = QPoint(QPoint(s->geometry().left(), s->geometry().center().y()) + QPoint(m_distance, m_offset));
-            break;
-        case Qt::AlignRight:
-            position = QPoint(s->geometry().bottomLeft() - QPoint(m_distance, m_offset + size().height()));
-            break;
-        case Qt::AlignLeft:
-        default:
-            position = QPoint(s->geometry().topLeft() + QPoint(m_distance, m_offset));
-        }
+        slideLocation = KWindowEffects::LeftEdge;
         break;
 
     case Plasma::Types::RightEdge:
         containment()->setFormFactor(Plasma::Types::Vertical);
-
-        switch (m_alignment) {
-        case Qt::AlignCenter:
-            position = QPoint(QPoint(s->geometry().right(), s->geometry().center().y()) - QPoint(width() + m_distance, 0) + QPoint(0, m_offset - size().height()/2));
-            break;
-        case Qt::AlignRight:
-            position = QPoint(s->geometry().bottomRight() - QPoint(width() + m_distance, 0) - QPoint(0, m_offset + size().height()));
-            break;
-        case Qt::AlignLeft:
-        default:
-            position = QPoint(s->geometry().topRight() - QPoint(width() + m_distance, 0) + QPoint(0, m_offset));
-        }
+        slideLocation = KWindowEffects::RightEdge;
         break;
 
     case Plasma::Types::BottomEdge:
     default:
         containment()->setFormFactor(Plasma::Types::Horizontal);
-
-        switch (m_alignment) {
-        case Qt::AlignCenter:
-            position = QPoint(QPoint(s->geometry().center().x(), s->geometry().bottom() - height() - m_distance) + QPoint(m_offset - size().width()/2, 1));
-            break;
-        case Qt::AlignRight:
-            position = QPoint(s->geometry().bottomRight() - QPoint(0, height() + m_distance) - QPoint(m_offset + size().width(), 1));
-            break;
-        case Qt::AlignLeft:
-        default:
-            position = QPoint(s->geometry().bottomLeft() - QPoint(0, height() + m_distance) + QPoint(m_offset, 1));
-        }
+        slideLocation = KWindowEffects::BottomEdge;
+        break;
     }
     m_strutsTimer.stop();
     m_strutsTimer.start(STRUTSTIMERDELAY);
     setMinimumSize(QSize(0, 0));
     setMaximumSize(screen()->size());
 
+    setGeometry(geometryByDistance(m_distance));
     if (formFactor() == Plasma::Types::Vertical) {
-        setGeometry(QRect(position, QSize(thickness(), length())));
         setMinimumSize(QSize(thickness(), m_minLength));
         setMaximumSize(QSize(thickness(), m_maxLength));
 
         emit thicknessChanged();
         emit lengthChanged();
     } else {
-        setGeometry(QRect(position, QSize(length(), thickness())));
         setMinimumSize(QSize(m_minLength, thickness()));
         setMaximumSize(QSize(m_maxLength, thickness()));
 
         emit thicknessChanged();
         emit lengthChanged();
     }
+    
+    KWindowEffects::slideWindow(winId(), slideLocation, -1);
+}
+
+QRect PanelView::geometryByDistance(int distance) const
+{
+    QScreen *s = screen();
+    QPoint position;
+    switch (containment()->location()) {
+    case Plasma::Types::TopEdge:
+        switch (m_alignment) {
+        case Qt::AlignCenter:
+            position = QPoint(QPoint(s->geometry().center().x(), s->geometry().top()) + QPoint(m_offset - size().width()/2, distance));
+            break;
+        case Qt::AlignRight:
+            position = QPoint(s->geometry().topRight() - QPoint(m_offset + size().width(), distance));
+            break;
+        case Qt::AlignLeft:
+        default:
+            position = QPoint(s->geometry().topLeft() + QPoint(m_offset, distance));
+        }
+        break;
+
+    case Plasma::Types::LeftEdge:
+        switch (m_alignment) {
+        case Qt::AlignCenter:
+            position = QPoint(QPoint(s->geometry().left(), s->geometry().center().y()) + QPoint(distance, m_offset));
+            break;
+        case Qt::AlignRight:
+            position = QPoint(s->geometry().bottomLeft() - QPoint(distance, m_offset + size().height()));
+            break;
+        case Qt::AlignLeft:
+        default:
+            position = QPoint(s->geometry().topLeft() + QPoint(distance, m_offset));
+        }
+        break;
+
+    case Plasma::Types::RightEdge:
+        switch (m_alignment) {
+        case Qt::AlignCenter:
+            position = QPoint(QPoint(s->geometry().right(), s->geometry().center().y()) - QPoint(width() + distance, 0) + QPoint(0, m_offset - size().height()/2));
+            break;
+        case Qt::AlignRight:
+            position = QPoint(s->geometry().bottomRight() - QPoint(width() + distance, 0) - QPoint(0, m_offset + size().height()));
+            break;
+        case Qt::AlignLeft:
+        default:
+            position = QPoint(s->geometry().topRight() - QPoint(width() + distance, 0) + QPoint(0, m_offset));
+        }
+        break;
+
+    case Plasma::Types::BottomEdge:
+    default:
+        switch (m_alignment) {
+        case Qt::AlignCenter:
+            position = QPoint(QPoint(s->geometry().center().x(), s->geometry().bottom() - height() - distance) + QPoint(m_offset - size().width()/2, 1));
+            break;
+        case Qt::AlignRight:
+            position = QPoint(s->geometry().bottomRight() - QPoint(0, height() + distance) - QPoint(m_offset + size().width(), 1));
+            break;
+        case Qt::AlignLeft:
+        default:
+            position = QPoint(s->geometry().bottomLeft() - QPoint(0, height() + distance) + QPoint(m_offset, 1));
+        }
+    }
+    return formFactor() == Plasma::Types::Vertical ? QRect(position, QSize(thickness(), length())) : QRect(position, QSize(length(), thickness()));
 }
 
 void PanelView::restore()
@@ -483,10 +480,9 @@ void PanelView::restore()
         m_offset = qMax(0, m_offset);
     }
 
+    setAlignment((Qt::Alignment)config().readEntry<int>("alignment", Qt::AlignLeft));
     setThickness(config().readEntry<int>("thickness", 30));
 
-    m_alignment = (Qt::Alignment)config().readEntry<int>("alignment", Qt::AlignLeft);
-
     setMinimumSize(QSize(-1, -1));
     //FIXME: an invalid size doesn't work with QWindows
     setMaximumSize(screen()->size());
@@ -543,6 +539,14 @@ void PanelView::showConfigurationInterface(Plasma::Applet *applet)
             KWindowSystem::setState(m_panelConfigView.data()->winId(), NET::SkipTaskbar | NET::SkipPager);
         }
         return;
+    } else if (m_panelConfigView) {
+        if (m_panelConfigView->applet() == applet) {
+            m_panelConfigView->show();
+            return;
+        } else {
+            m_panelConfigView->hide();
+            m_panelConfigView->deleteLater();
+        }
     }
 
     if (cont && cont->isContainment()) {
@@ -550,6 +554,7 @@ void PanelView::showConfigurationInterface(Plasma::Applet *applet)
     } else {
         m_panelConfigView = new PlasmaQuick::ConfigView(applet);
     }
+
     m_panelConfigView.data()->init();
     m_panelConfigView.data()->show();
     KWindowSystem::setState(m_panelConfigView.data()->winId(), NET::SkipTaskbar | NET::SkipPager);
@@ -615,48 +620,98 @@ void PanelView::setAutoHideEnabled(bool enabled)
 
 void PanelView::resizeEvent(QResizeEvent *ev)
 {
-    if (containment()) {
-        if (containment()->formFactor() == Plasma::Types::Vertical) {
-            config().writeEntry("length", ev->size().height());
-            if (ev->size().height() != ev->oldSize().height()) {
-                emit lengthChanged();
-            }
-            if (ev->size().width() != ev->oldSize().width()) {
-                emit thicknessChanged();
-            }
-        } else {
-            config().writeEntry("length", ev->size().width());
-            if (ev->size().width() != ev->oldSize().width()) {
-                emit lengthChanged();
-            }
-            if (ev->size().height() != ev->oldSize().height()) {
-                emit thicknessChanged();
-            }
-        }
-
-        m_positionPaneltimer.start();
-    }
+    updateMask();
     PlasmaQuick::View::resizeEvent(ev);
 }
 
+void PanelView::moveEvent(QMoveEvent *ev)
+{
+    updateMask();
+    PlasmaQuick::View::moveEvent(ev);
+}
+
+void PanelView::integrateScreen()
+{
+    KWindowSystem::setOnAllDesktops(winId(), true);
+    KWindowSystem::setType(winId(), NET::Dock);
+    setVisibilityMode(m_visibilityMode);
+}
+
 void PanelView::showEvent(QShowEvent *event)
 {
     PanelShadows::self()->addWindow(this);
     PlasmaQuick::View::showEvent(event);
-    KWindowSystem::setOnAllDesktops(winId(), true);
-    KWindowSystem::setType(winId(), NET::Dock);
+    integrateScreen();
+
+    //When the screen is set, the screen is recreated internally, so we need to
+    //set anything that depends on the winId()
+    connect(this, &QWindow::screenChanged, this, [=]() {
+        if (!screen())
+            return;
+        integrateScreen();
+        showTemporarily();
+    });
 }
 
 bool PanelView::event(QEvent *e)
 {
-    if (e->type() == QEvent::Enter) {
-        m_unhideTimer.stop();
-    } else if (e->type() == QEvent::Leave) {
-        m_unhideTimer.start();
+    if (m_visibilityMode == AutoHide) {
+        if (e->type() == QEvent::Enter) {
+            m_unhideTimer.stop();
+        } else if (e->type() == QEvent::Leave) {
+            m_unhideTimer.start();
+        }
     }
     return View::event(e);
 }
 
+void PanelView::updateMask()
+{
+    if (KWindowSystem::compositingActive()) {
+        setMask(QRegion());
+    } else {
+        if (!m_background) {
+            m_background = new Plasma::FrameSvg(this);
+            m_background->setImagePath("widgets/panel-background");
+        }
+
+        Plasma::FrameSvg::EnabledBorders borders = Plasma::FrameSvg::AllBorders;
+        switch (location()) {
+        case Plasma::Types::TopEdge:
+            borders &= ~Plasma::FrameSvg::TopBorder;
+            break;
+        case Plasma::Types::LeftEdge:
+            borders &= ~Plasma::FrameSvg::LeftBorder;
+            break;
+        case Plasma::Types::RightEdge:
+            borders &= ~Plasma::FrameSvg::RightBorder;
+            break;
+        case Plasma::Types::BottomEdge:
+            borders &= ~Plasma::FrameSvg::BottomBorder;
+            break;
+        default:
+            break;
+        }
+
+        if (x() <= screen()->geometry().x()) {
+            borders &= ~Plasma::FrameSvg::LeftBorder;
+        }
+        if (x() + width() >= screen()->geometry().x() + screen()->geometry().width()) {
+            borders &= ~Plasma::FrameSvg::RightBorder;
+        }
+        if (y() <= screen()->geometry().y()) {
+            borders &= ~Plasma::FrameSvg::TopBorder;
+        }
+        if (y() + height() >= screen()->geometry().y() + screen()->geometry().height()) {
+            borders &= ~Plasma::FrameSvg::BottomBorder;
+        }
+        m_background->setEnabledBorders(borders);
+
+        m_background->resizeFrame(size());
+        setMask(m_background->mask());
+    }
+}
+
 void PanelView::updateStruts()
 {
     if (!containment() || !screen()) {
@@ -769,12 +824,13 @@ void PanelView::updateStruts()
 
 void PanelView::themeChanged()
 {
-    //TODO: how to take the shape from the framesvg?
     KWindowEffects::enableBlurBehind(winId(), true);
     KWindowEffects::enableBackgroundContrast(winId(), m_theme.backgroundContrastEnabled(),
                                                       m_theme.backgroundContrast(),
                                                       m_theme.backgroundIntensity(),
                                                       m_theme.backgroundSaturation());
+
+    updateMask();
 }
 
 void PanelView::containmentChanged()
@@ -804,4 +860,15 @@ void PanelView::showTemporarily()
     t->start();
 }
 
+void PanelView::screenDestroyed(QObject* screen)
+{
+//     NOTE: this is overriding the screen destroyed slot, we need to do this because
+//     otherwise Qt goes mental and starts moving our panels. See:
+//     https://codereview.qt-project.org/#/c/88351/
+//     if(screen == this->screen()) {
+//         DO NOTHING, panels are moved by ::removeScreen
+//     }
+}
+
+
 #include "moc_panelview.cpp"
