@@ -83,10 +83,17 @@ public:
     VisibilityMode visibilityMode() const;
     void setVisibilityMode(PanelView::VisibilityMode mode);
 
+    /**
+     * @returns the geometry of the panel given a distance
+     */
+    QRect geometryByDistance(int distance) const;
+
 protected:
     void resizeEvent(QResizeEvent *ev);
     void showEvent(QShowEvent *event);
+    void moveEvent(QMoveEvent *ev);
     bool event(QEvent *e);
+    void updateMask();
 
 Q_SIGNALS:
     void alignmentChanged();
@@ -118,8 +125,11 @@ private Q_SLOTS:
     void containmentChanged();
     void statusChanged(Plasma::Types::ItemStatus);
     void restoreAutoHide();
+    void screenDestroyed(QObject* screen);
 
 private:
+    void integrateScreen();
+
     int m_offset;
     int m_maxLength;
     int m_minLength;
@@ -133,6 +143,8 @@ private:
     Plasma::Theme m_theme;
     QTimer m_positionPaneltimer;
     QTimer m_unhideTimer;
+    //only for the mask, not to actually paint
+    Plasma::FrameSvg *m_background;
 
     static const int STRUTSTIMERDELAY = 200;
 };
