@@ -41,6 +41,7 @@
 #include <KAuthorized>
 #include <KWindowSystem>
 #include <kdeclarative/kdeclarative.h>
+#include <KMessageBox>
 
 #include <KScreen/Config>
 #include <kscreen/configmonitor.h>
@@ -97,6 +98,7 @@ public:
     QAction *addPanelAction;
     QMenu *addPanelsMenu;
     Plasma::Package lookNFeelPackage;
+    QSet<KScreen::Output*> redundantOutputs;
 #if HAVE_KTEXTEDITOR
     QWeakPointer<InteractiveConsole> console;
 #endif
@@ -152,6 +154,11 @@ ShellCorona::ShellCorona(QObject *parent)
     // Look for theme config in plasmarc, if it isn't configured, take the theme from the
     // LookAndFeel package, if either is set, change the default theme
 
+    connect(qApp, &QCoreApplication::aboutToQuit, [=]() {
+        //saveLayout is a slot but arguments not compatible
+        saveLayout();
+    });
+
     const QString themeGroupKey = QStringLiteral("Theme");
     const QString themeNameKey = QStringLiteral("name");
 
@@ -254,8 +261,10 @@ QString ShellCorona::shell() const
 
 bool outputLess(KScreen::Output *a, KScreen::Output *b)
 {
-    return (a->isPrimary() == b->isPrimary() || (a->isPrimary() && !b->isPrimary()))
-        && (a->pos().x() < b->pos().x() || (a->pos().x() == b->pos().x() && a->pos().y() < b->pos().y()));
+    return ((a->isEnabled() && !b->isEnabled())
+         || (a->isEnabled() == b->isEnabled() && (a->isPrimary() && !b->isPrimary()))
+         || (a->isPrimary() == b->isPrimary() && (a->pos().x() < b->pos().x()
+         || (a->pos().x() == b->pos().x() && a->pos().y() < b->pos().y()))));
 }
 
 static QList<KScreen::Output*> sortOutputs(const QHash<int, KScreen::Output*> &outputs)
@@ -302,7 +311,8 @@ void ShellCorona::load()
     }
 
     KScreen::ConfigMonitor::instance()->addConfig(d->screenConfiguration);
-    for (KScreen::Output *output : sortOutputs(d->screenConfiguration->connectedOutputs())) {
+    //we're not going through the connectedOutputs because we need to connect to all outputs
+    for (KScreen::Output *output : sortOutputs(d->screenConfiguration->outputs())) {
         addOutput(output);
     }
     connect(d->screenConfiguration, &KScreen::Config::outputAdded, this, &ShellCorona::addOutput);
@@ -317,7 +327,7 @@ void ShellCorona::load()
 void ShellCorona::primaryOutputChanged()
 {
     KScreen::Config *current = d->screenConfiguration;
-    if (!current->primaryOutput())
+    if (!current->primaryOutput() || !current->primaryOutput()->isEnabled())
         return;
     QScreen *newPrimary = outputToScreen(current->primaryOutput());
     int i=0;
@@ -327,12 +337,12 @@ void ShellCorona::primaryOutputChanged()
         i++;
     }
     QScreen *oldPrimary = d->views.first()->screen();
-    qDebug() << "primary changed!" << oldPrimary->name() << newPrimary->name() << i;
-//
-//     //if it was not found, it means that addOutput hasn't been called yet
+
+    //if it was not found, it means that addOutput hasn't been called yet
     if (i>=d->views.count() || i==0)
         return;
-//
+    qDebug() << "primary changed!" << oldPrimary->name() << newPrimary->name() << i;
+
     Q_ASSERT(oldPrimary != newPrimary);
     Q_ASSERT(d->views[0]->screen() != d->views[i]->screen());
     Q_ASSERT(d->views[0]->screen() == oldPrimary);
@@ -357,7 +367,7 @@ void ShellCorona::primaryOutputChanged()
 void ShellCorona::screenInvariants() const
 {
     Q_ASSERT(d->views.count() <= QGuiApplication::screens().count());
-    Q_ASSERT(!d->screenConfiguration->primaryOutput() || !d->views.isEmpty() || outputToScreen(d->screenConfiguration->primaryOutput()) == d->views.first()->screen());
+    Q_ASSERT(!outputToScreen(d->screenConfiguration->primaryOutput()) || d->views.isEmpty() || outputToScreen(d->screenConfiguration->primaryOutput()) == d->views.first()->screen() || !d->screenConfiguration->primaryOutput()->isEnabled());
     QScreen *s = d->views.isEmpty() ? nullptr : d->views[0]->screen();
     if (!s) {
         qWarning() << "error: couldn't find primary output" << d->screenConfiguration->primaryOutput();
@@ -365,14 +375,18 @@ void ShellCorona::screenInvariants() const
     }
 
     Q_ASSERT(d->views[0]->screen()->name() == s->name());
-    if (!ShellManager::s_forceWindowed) {
-        Q_ASSERT(d->views[0]->geometry() == s->geometry());
-    }
+    Q_ASSERT(d->views[0]->geometry() == s->geometry() || ShellManager::s_forceWindowed);
 
     QSet<QScreen*> screens;
+    int i = 0;
     foreach(DesktopView *view, d->views) {
         Q_ASSERT(!screens.contains(view->screen()));
+        Q_ASSERT(view->isVisible());
+        Q_ASSERT(view->fillScreen() || ShellManager::s_forceWindowed);
+        Q_ASSERT(view->containment()->screen() == i);
+
         screens.insert(view->screen());
+        ++i;
     }
 }
 
@@ -442,128 +456,83 @@ KActivities::Controller *ShellCorona::activityController()
 
 int ShellCorona::numScreens() const
 {
-    return QGuiApplication::screens().count();
+    return d->views.count();
 }
 
 QRect ShellCorona::screenGeometry(int id) const
 {
-    DesktopView *view = 0;
-    foreach (DesktopView *v, d->views) {
-        if (v->containment() && v->containment()->screen() == id) {
-            view = v;
-            break;
-        }
+    if (id>=d->views.count() || id<0) {
+        qWarning() << "requesting unexisting screen" << id;
+        QScreen *s = outputToScreen(d->screenConfiguration->primaryOutput());
+        return s ? s->geometry() : QRect();
     }
-
-    if (view) {
-        return view->geometry();
-    }
-
-    //each screen should have a view
-    qWarning() << "requesting unexisting screen" << id;
-    QScreen *s = outputToScreen(d->screenConfiguration->primaryOutput());
-    return s ? s->geometry() : QRect();
+    return d->views[id]->geometry();
 }
 
 QRegion ShellCorona::availableScreenRegion(int id) const
 {
-    const QRect screenGeo(screenGeometry(id));
-
-    DesktopView *view = 0;
-    if (id >= 0 && id < d->views.count()) {
-        view = d->views[id];
-    }
-
-    if (view) {
-        QRegion r = view->geometry();
-        QRect panelRect;
-        foreach (PanelView *v, d->panelViews) {
-            panelRect = v->geometry();
-            if (v->containment()->screen() == id && v->visibilityMode() != PanelView::AutoHide) {
-                //don't use panel positions, because sometimes the panel can be moved around
-                switch (view->location()) {
-                case Plasma::Types::TopEdge:
-                    panelRect.moveTop(qMin(panelRect.top(), screenGeo.top()));
-                    break;
-
-                case Plasma::Types::BottomEdge:
-                    panelRect.moveBottom(qMax(panelRect.bottom(), screenGeo.bottom()));
-                    break;
-
-                case Plasma::Types::LeftEdge:
-                    panelRect.moveLeft(qMin(panelRect.left(), screenGeo.left()));
-                    break;
-
-                case Plasma::Types::RightEdge:
-                    panelRect.moveRight(qMax(panelRect.right(), screenGeo.right()));
-                    break;
-
-                default:
-                    break;
-                }
-                r -= panelRect;
-            }
-        }
-        return r;
-    } else {
+    if (id>=d->views.count() || id<0) {
         //each screen should have a view
         qWarning() << "requesting unexisting screen" << id;
         QScreen *s = outputToScreen(d->screenConfiguration->primaryOutput());
         return s ? s->availableGeometry() : QRegion();
     }
+    DesktopView *view = d->views[id];
+    const QRect screenGeo(view->geometry());
+
+    QRegion r = view->geometry();
+    foreach (PanelView *v, d->panelViews) {
+        if (v->screen() == v->screen() && v->visibilityMode() != PanelView::AutoHide) {
+            //if the panel is being moved around, we still want to calculate it from the edge
+            r -= v->geometryByDistance(0);
+        }
+    }
+    return r;
 }
 
+int rectArea(const QRect& r) { return r.width()*r.height(); }
+
 QRect ShellCorona::availableScreenRect(int id) const
 {
-    if (id < 0) {
-        id = 0;
+    if (id >= d->views.count() || id < 0) {
+        //each screen should have a view
+        qWarning() << "requesting unexisting screen" << id;
+        QScreen *s = outputToScreen(d->screenConfiguration->primaryOutput());
+        return s ? s->availableGeometry() : QRect();
     }
 
-    const QRect screenGeo(screenGeometry(id));
-    QRect r(screenGeometry(id));
+    DesktopView *view = d->views[id];
+    const QRect screenGeo(view->geometry());
 
-    foreach (PanelView *view, d->panelViews) {
-        if (view->containment()->screen() == id && view->visibilityMode() != PanelView::AutoHide) {
-            QRect v = view->geometry();
-            switch (view->location()) {
-                case Plasma::Types::TopEdge:
-                    //neutralize eventual panel positioning due to drag handle
-                    v.moveTop(qMin(v.top(), screenGeo.top()));
-                    if (v.bottom() > r.top()) {
-                        r.setTop(v.bottom());
-                    }
-                    break;
-
-                case Plasma::Types::BottomEdge:
-                    v.moveBottom(qMax(v.bottom(), screenGeo.bottom()));
-                    if (v.top() < r.bottom()) {
-                        r.setBottom(v.top());
-                    }
-                    break;
-
-                case Plasma::Types::LeftEdge:
-                    v.moveLeft(qMin(v.left(), screenGeo.left()));
-                    if (v.right() > r.left()) {
-                        r.setLeft(v.right());
-                    }
-                    break;
-
-                case Plasma::Types::RightEdge:
-                    v.moveRight(qMax(v.right(), screenGeo.right()));
-                    if (v.left() < r.right()) {
-                        r.setRight(v.left());
-                    }
-                    break;
-
-                default:
-                    break;
+    QRect r = view->geometry();
+    foreach (PanelView *v, d->panelViews) {
+        if (v->screen() == v->screen() && v->visibilityMode() != PanelView::AutoHide) {
+            switch (v->location()) {
+            case Plasma::Types::LeftEdge:
+                r.setLeft(r.left() + v->thickness());
+                break;
+            case Plasma::Types::RightEdge:
+                r.setRight(r.right() - v->thickness());
+                break;
+            case Plasma::Types::TopEdge:
+                r.setTop(r.top() + v->thickness());
+                break;
+            case Plasma::Types::BottomEdge:
+                r.setBottom(r.bottom() - v->thickness());
+            default:
+                break;
             }
         }
     }
-
     return r;
 }
 
+QScreen* ShellCorona::screenForId(int screenId) const
+{
+    DesktopView* v = d->views.value(screenId);
+    return v ? v->screen() : Q_NULLPTR;
+}
+
 PanelView *ShellCorona::panelView(Plasma::Containment *containment) const
 {
     return d->panelViews.value(containment);
@@ -571,11 +540,35 @@ PanelView *ShellCorona::panelView(Plasma::Containment *containment) const
 
 ///// SLOTS
 
-void ShellCorona::shiftViews(int idx, int delta, int until)
+QList<PanelView*> ShellCorona::panelsForScreen(QScreen* screen)
 {
-    for (int i = idx; i<until; ++i) {
-        d->views[i]->setScreen(d->views[i-delta]->screen());
-        d->views[i]->adaptToScreen();
+    QList<PanelView*> ret;
+    foreach(PanelView* v, d->panelViews) {
+        if(v->screen() == screen)
+            ret += v;
+    }
+    return ret;
+}
+
+void ShellCorona::removeView(int idx)
+{
+    if (idx == d->views.count()-1) {
+        QScreen* screen = d->views[idx]->screen();
+        QList<PanelView*> panels = panelsForScreen(screen);
+
+        foreach(PanelView* p, panels) {
+            delete d->panelViews.take(p->containment());
+        }
+        delete d->views.takeLast();
+    } else if (idx < d->views.count()-1) {
+        QScreen* screen = d->views[idx+1]->screen();
+        QList<PanelView*> panels = panelsForScreen(screen);
+
+        d->views[idx]->setScreen(screen);
+        foreach(PanelView* p, panels) {
+            p->setScreen(screen);
+        }
+        removeView(idx+1);
     }
 }
 
@@ -584,23 +577,65 @@ void ShellCorona::outputEnabledChanged()
     addOutput(qobject_cast<KScreen::Output*>(sender()));
 }
 
-void ShellCorona::addOutput(KScreen::Output *output)
+bool ShellCorona::isOutputRedundant(KScreen::Output* screen) const
 {
-    connect(output, &KScreen::Output::isEnabledChanged, this, &ShellCorona::outputEnabledChanged, Qt::UniqueConnection);
-    if (!output->isEnabled())
-        return;
-    QScreen *screen = outputToScreen(output);
-    Q_ASSERT(screen);
+    QRect geometry = screen->geometry();
 
     //FIXME: QScreen doesn't have any idea of "this qscreen is clone of this other one
     //so this ultra inefficient heuristic has to stay until we have a slightly better api
-    foreach (QScreen *s, QGuiApplication::screens()) {
-        if (s->geometry().contains(screen->geometry(), false) &&
-            s->geometry().width() > screen->geometry().width() &&
-            s->geometry().height() > screen->geometry().height()) {
-            return;
+    foreach (KScreen::Output *s, d->screenConfiguration->connectedOutputs()) {
+        if (!s->isEnabled())
+            continue;
+        QRect sGeometry = s->geometry();
+        if (sGeometry.contains(geometry, false) &&
+            sGeometry.width() > geometry.width() &&
+            sGeometry.height() > geometry.height()) {
+            return true;
         }
     }
+    return false;
+}
+
+void ShellCorona::reconsiderOutputs()
+{
+    KScreen::Output* output = qobject_cast<KScreen::Output*>(sender());
+    reconsiderOutput(output);
+    foreach(KScreen::Output* out, d->redundantOutputs) {
+        reconsiderOutput(out);
+    }
+}
+
+void ShellCorona::reconsiderOutput(KScreen::Output* output)
+{
+    QScreen *screen = outputToScreen(output);
+    foreach(DesktopView* view, d->views) {
+        if (view->screen() == screen) {
+            if (isOutputRedundant(output)) { //if redundant, then remove the screen
+                removeScreen(view);
+            }
+            return; //if it's already in, let's not add it again
+        }
+    }
+    addOutput(output);
+}
+
+void ShellCorona::addOutput(KScreen::Output *output)
+{
+    connect(output, &KScreen::Output::isEnabledChanged, this, &ShellCorona::outputEnabledChanged, Qt::UniqueConnection);
+    connect(output, &KScreen::Output::posChanged, this, &ShellCorona::reconsiderOutputs, Qt::UniqueConnection);
+    connect(output, &KScreen::Output::currentModeIdChanged, this, &ShellCorona::reconsiderOutputs, Qt::UniqueConnection);
+    if (!output->isEnabled()) {
+        d->redundantOutputs.remove(output);
+        return;
+    }
+    QScreen *screen = outputToScreen(output);
+    Q_ASSERT(screen);
+
+    if (isOutputRedundant(output)) {
+        d->redundantOutputs.insert(output);
+        return;
+    } else
+        d->redundantOutputs.remove(output);
 
     int insertPosition = 0;
     foreach (DesktopView *view, d->views) {
@@ -611,22 +646,13 @@ void ShellCorona::addOutput(KScreen::Output *output)
         insertPosition++;
     }
 
-    DesktopView *view = new DesktopView(this, screen);
+    QScreen* newScreen = insertScreen(screen, insertPosition);
 
-    //We have to do it in a lambda,
-    connect(screen, &QObject::destroyed, this, [=]() { removeDesktop(view); });
+    DesktopView *view = new DesktopView(this, newScreen);
+    d->views.append(view);
 
-    d->views.insert(insertPosition, view);
-    shiftViews(insertPosition+1, 1, d->views.count()-1);
-
-    const QString currentActivity = d->activityController->currentActivity();
-    Plasma::Containment *containment = d->desktopContainments[currentActivity].value(insertPosition);
-    if (!containment) {
-        containment = createContainmentForActivity(currentActivity, insertPosition);
-    }
-    if (!containment) {
-        return;
-    }
+    Plasma::Containment *containment = createContainmentForActivity(d->activityController->currentActivity(), d->views.count()-1);
+    Q_ASSERT(containment);
 
     QAction *removeAction = containment->actions()->action("remove");
     if (removeAction) {
@@ -634,8 +660,6 @@ void ShellCorona::addOutput(KScreen::Output *output)
     }
 
     view->setContainment(containment);
-
-   // connect(screen, SIGNAL(destroyed(QObject*)), SLOT(screenRemoved(QObject*)));
     view->show();
 
     //were there any panels for this screen before it popped up?
@@ -649,24 +673,47 @@ void ShellCorona::addOutput(KScreen::Output *output)
     screenInvariants();
 }
 
-void ShellCorona::removeDesktop(DesktopView *view)
+QScreen* ShellCorona::insertScreen(QScreen* screen, int idx)
 {
-    int idx = d->views.indexOf(view);
-    DesktopView *lastView = d->views.takeAt(d->views.count()-1);
-    lastView->deleteLater();
+    if(idx==d->views.count())
+        return screen;
 
-    shiftViews(idx, -1, d->views.count()-1);
+    DesktopView* v = d->views[idx];
+    QScreen *oldScreen = v->screen();
+    v->setScreen(screen);
+    v->adaptToScreen();
+    foreach(PanelView* panel, d->panelViews) {
+        if(panel->screen() == screen) {
+            panel->setScreen(screen);
+        }
+    }
+
+    return insertScreen(oldScreen, idx+1);
+}
+
+void ShellCorona::removeScreen(DesktopView *view)
+{
+    foreach(PanelView* panel, d->panelViews) {
+        if (panel->screen() == view->screen())
+            removePanel(panel);
+    }
+
+    int idx = d->views.indexOf(view);
+
+    removeView(idx);
 
     screenInvariants();
 }
 
-void ShellCorona::removePanel(QObject *c)
+void ShellCorona::removePanel(PanelView* panelView)
 {
-    Plasma::Containment *cont = qobject_cast<Plasma::Containment*>(c);
-
-    d->panelViews[cont]->deleteLater();
+    Plasma::Containment* cont = panelView->containment();
     d->waitingPanels << cont;
     d->panelViews.remove(cont);
+    delete panelView;
+
+    emit availableScreenRectChanged();
+    emit availableScreenRegionChanged();
 }
 
 Plasma::Containment *ShellCorona::createContainmentForActivity(const QString& activity, int screenNum)
@@ -707,16 +754,17 @@ void ShellCorona::createWaitingPanels()
             continue;
         }
 
-        d->panelViews[cont] = new PanelView(this);
+        PanelView* panel = new PanelView(this);
+        d->panelViews[cont] = panel;
 
         Q_ASSERT(qBound(0, requestedScreen, d->views.size() -1) == requestedScreen);
         QScreen *screen = d->views[requestedScreen]->screen();
 
-        d->panelViews[cont]->setScreen(screen);
-        d->panelViews[cont]->setContainment(cont);
+        panel->setScreen(screen);
+        panel->setContainment(cont);
+        panel->show();
 
         connect(cont, SIGNAL(destroyed(QObject*)), this, SLOT(containmentDeleted(QObject*)));
-        connect(screen, SIGNAL(destroyed(QObject*)), this, SLOT(removePanel(QObject*)));
     }
     d->waitingPanels = stillWaitingPanels;
     emit availableScreenRectChanged();
@@ -1085,6 +1133,7 @@ Plasma::Containment *ShellCorona::addPanel(const QString &plugin)
         break;
     }
 
+    Q_ASSERT(panel);
     d->waitingPanels << panel;
     createWaitingPanels();
 
@@ -1101,6 +1150,15 @@ Plasma::Containment *ShellCorona::addPanel(const QString &plugin)
 
 int ShellCorona::screenForContainment(const Plasma::Containment *containment) const
 {
+    //case in which this containment is child of an applet, hello systray :)
+    if (Plasma::Applet *parentApplet = qobject_cast<Plasma::Applet *>(containment->parent())) {
+        if (Plasma::Containment* cont = parentApplet->containment()) {
+            return screenForContainment(cont);
+        } else {
+            return -1;
+        }
+    }
+
     for (int i = 0; i < d->views.size(); i++) {
         if (d->views[i]->containment() == containment) {
             return i;
@@ -1206,6 +1264,21 @@ KScreen::Config* ShellCorona::screensConfiguration() const
     return d->screenConfiguration;
 }
 
+void ShellCorona::showOpenGLNotCompatibleWarning()
+{
+    static bool s_multipleInvokations = false;
+    if (s_multipleInvokations) {
+        return;
+    }
+    s_multipleInvokations = true;
+    KMessageBox::error(nullptr,
+                       i18n("Your graphics hardware does not support OpenGL (ES) 2. Plasma will abort now."),
+                       i18n("Incompatible OpenGL version detected")
+                      );
+    // this doesn't work and I have no idea why.
+    QCoreApplication::exit(1);
+}
+
 // Desktop corona handler
 
 
