@@ -80,6 +80,9 @@ public:
 
     KScreen::Config* screensConfiguration() const;
 
+    void removePanel(PanelView* panelView);
+    QScreen* screenForId(int screenId) const;
+
 public Q_SLOTS:
     /**
      * Request saving applicationConfig on disk, it's event compressed, not immediate
@@ -104,6 +107,7 @@ public Q_SLOTS:
     void toggleActivityManager();
 
     Plasma::Containment *addPanel(const QString &plugin);
+    void removeScreen(DesktopView* screen);
 
 protected Q_SLOTS:
     /**
@@ -143,8 +147,6 @@ private Q_SLOTS:
     void containmentDeleted(QObject* cont);
 
     void outputEnabledChanged();
-    void removePanel(QObject* cont);
-    void removeDesktop(DesktopView* screen);
     void addOutput(KScreen::Output* output);
     void primaryOutputChanged();
 
@@ -152,10 +154,16 @@ private Q_SLOTS:
     void activityClosed();
     void activityRemoved();
     void desktopContainmentDestroyed(QObject*);
+    void showOpenGLNotCompatibleWarning();
 
 private:
-    void shiftViews(int idx, int delta, int until);
+    QScreen* insertScreen(QScreen* screen, int idx);
+    void removeView(int idx);
     void screenInvariants() const;
+    bool isOutputRedundant(KScreen::Output* output) const;
+    void reconsiderOutput(KScreen::Output* output);
+    void reconsiderOutputs();
+    QList<PanelView*> panelsForScreen(QScreen* screen);
 
     /**
      * @returns a new containment associated with the specified @p activity and @p screen.
