@@ -140,7 +140,6 @@ void WidgetExplorerPrivate::initFilters()
         if (existingCategories.contains(lowerCaseCat)) {
             const QString trans = i18n(category.toLocal8Bit());
             categories.insert(trans.toLower(), qMakePair(trans, lowerCaseCat));
-            qDebug() << "Categories: << " << lowerCaseCat;
         }
     }
 
@@ -223,7 +222,7 @@ void WidgetExplorerPrivate::initRunningApplets()
     //we've tried our best to get a corona
     //we don't want just one containment, we want them all
     if (!c) {
-        qDebug() << "can't happen";
+        qWarning() << "WidgetExplorer failed to find corona";
         return;
     }
 
@@ -401,6 +400,7 @@ void WidgetExplorer::downloadWidgets(const QString &type)
 {
     if (!d->newStuffDialog) {
         d->newStuffDialog = new KNS3::DownloadDialog( QString::fromLatin1("plasmoids.knsrc") );
+        d->newStuffDialog.data()->setWindowTitle(i18n("Download New Plasma Widgets"));
         connect(d->newStuffDialog.data(), SIGNAL(accepted()), SLOT(newStuffFinished()));
     }
     d->newStuffDialog.data()->show();
